import logging
import os
import sys
import subprocess
import re

from pyanaconda.core import util
from pyanaconda.modules.common.task import Task

from cern_customizations.constants import SYSCONFIG_FILE_PATH, \
            AFS_IS_DEFAULT, EOS_IS_DEFAULT, CVMFS_IS_DEFAULT, \
            AUTOUPDATE_IS_DEFAULT, RUN_IS_DEFAULT, LOCMAP_BIN, \
            SYSTEMCTL_BIN, SYSCONFIG_YUM_PATH, \
            LOCMAP_CORE_MODULES, AUTOUPDATEBOOT_IS_DEFAULT, \
            GNOME_INIT_DEFAULT, GDMCONFIG_FILE_PATH, \
            LOCMAP_SYSTEMD_UNIT, XDGCONFIG_FILE_PATH


# C8
if (sys.version_info > (3, 0)):
  try:
    from pyanaconda.core.configuration.anaconda import conf
    system_root = conf.target.system_root
  # older versions
  except:
    from pyanaconda.core.util import getSysroot
    system_root = getSysroot()
# CC7
else:
  from pyanaconda.iutil import getSysroot
  system_root = getSysroot()

log = logging.getLogger(__name__)

__all__ = [ "CernCustomizationsInstallationTask"]

class CernCustomizationsInstallationTask(Task):

    def __init__(self, run, afs, autoupdate, cvmfs, gnomeinit):
        """Create a task."""
        super().__init__()
        self._run = run
        self._afs = afs
        self._autoupdate = autoupdate
        self._cvmfs = cvmfs
        self._gnomeinit = gnomeinit

    @property
    def name(self):
        return "Run CERN customizations"

    def run(self):
        """Run the task."""

        yumupdate_file = os.path.normpath(system_root + SYSCONFIG_YUM_PATH)
        # Data is persisted in /root/initial-setup-ks.cfg
        sysconfig_file = os.path.normpath(system_root + SYSCONFIG_FILE_PATH)
        with open(sysconfig_file, "w") as fobj:
            fobj.write("LOCMAP_FIRSTBOOT_START=%s\n" % self._run)
            # For compatibility but can be dropped in >= 7.5
            fobj.write("LOCMAP_FIRSTBOOT_STARTAFS=%s\n" % self._afs)
            #fobj.write("LOCMAP_FIRSTBOOT_EOSCLIENT=%s\n" % self.eos)
            fobj.write("LOCMAP_FIRSTBOOT_CVMFS=%s\n" % self._cvmfs)
            fobj.write("LOCMAP_FIRSTBOOT_UPDATE=%s\n" % self._autoupdate)
            fobj.close()

        # Allow to disable gnome-initial-setup
        gdmconfig_file = os.path.normpath(system_root + GDMCONFIG_FILE_PATH)
        xdgconfig_file = os.path.normpath(system_root + XDGCONFIG_FILE_PATH)
        if True:
        #FIX FIX FIX FIX if not self._gnomeinit:
            # C8
            if (sys.version_info > (3, 0)):
                from configparser import SafeConfigParser, NoOptionError, NoSectionError
            # CC7
            else:
                from ConfigParser import SafeConfigParser, NoOptionError, NoSectionError
            config = SafeConfigParser()
            config.optionxform = str
            config.read(gdmconfig_file)
            try:
                config.set('daemon', 'InitialSetupEnable', 'False')
                with open(gdmconfig_file, "w") as fobj:
                    config.write(fobj)
                    fobj.close()
            except (NoOptionError,NoSectionError):
                print("Unexpected error:", sys.exc_info()[0])

            configxdg = SafeConfigParser()
            configxdg.optionxform = str
            configxdg.read(xdgconfig_file)
            try:
                configxdg.set('Desktop Entry', 'X-GNOME-Autostart-enabled', 'false')
                with open(xdgconfig_file, "w") as fobj:
                    configxdg.write(fobj)
                    fobj.close()
            except (NoOptionError,NoSectionError):
                print("Unexpected error:", sys.exc_info()[0])

        # we will need this for later
        release_string=open('/etc/redhat-release','r').read().rstrip()
        release = re.search('\d+',release_string).group()

        # Configure yum-autoupdate
        if self._autoupdate:
            if release >= '8':
                subprocess.call(["dnf", "-y", "install", "yum-autoupdate"])
            else:
                yumautoupdate_installed=os.path.isfile('/usr/sbin/yum-autoupdate')
                if yumautoupdate_installed == False:
                    subprocess.call(["yum", "-y", "install", "yum-autoupdate"])
                subprocess.call([SYSTEMCTL_BIN, "enable", "yum-autoupdate"])
                try:
                    with open(yumupdate_file, "a") as fobj:
                        fobj.write("\n# initial-setup [cern-anaconda-cern] values\n")
                        fobj.write("YUMUPDATE=%d\n" % int(self._autoupdate))
                        fobj.close()
                except IOError:
                    print("I/O Error couldn't write %s" % (yumupdate_file))
                except:
                    print("Unexpected error:", sys.exc_info()[0])
        else:
            # we use call rather than check_call as we don't care if this fails
            # due to yumautoupdate not being installed
            if release >= '8':
                subprocess.call([SYSTEMCTL_BIN, "disable", "dnf-automatic-install.timer"])
            else:
                subprocess.call([SYSTEMCTL_BIN, "disable", "yum-autoupdate"])

        # locmap may not be installed - force install if it's not
        locmap_installed=os.path.isfile('/usr/bin/locmap')
        if locmap_installed == False:
            # locmap does not reside in CERN repo on 8, install locmap-release
            if release >= '8':
                subprocess.call(["dnf", "-y", "install", "locmap-release"])
            # 'yum' will work for both 7 and 8
            subprocess.call(["yum", "-y", "install", "locmap"])

        # the above call to install locmap could have failed if the host is
        # outside of CERN...
        locmap_installed=os.path.isfile('/usr/bin/locmap')
        if locmap_installed == True:
            # Enable locmap core modules by default
            for module in LOCMAP_CORE_MODULES:
                subprocess.call([LOCMAP_BIN, "--enable", module])

          # Configure and run locmap only if not disabled
            if self.run:
                subprocess.call([SYSTEMCTL_BIN, "enable", LOCMAP_SYSTEMD_UNIT])
                try:
                    if self._afs:
                        subprocess.check_call([LOCMAP_BIN, "--enable", "afs"])
                    else:
                        subprocess.check_call([LOCMAP_BIN, "--disable", "afs"])
                except subprocess.CalledProcessError:
                    print("Couldn't run locmap successfully")
                except:
                    print("Unexpected error:", sys.exc_info()[0])
                try:
                    if self._cvmfs:
                        subprocess.check_call([LOCMAP_BIN, "--enable", "cvmfs"])
                    else:
                        subprocess.check_call([LOCMAP_BIN, "--disable", "cvmfs"])
                except subprocess.CalledProcessError:
                    print("Couldn't run locmap successfully")
                except:
                    print("Unexpected error:", sys.exc_info()[0])
